-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to set up a timer to auto-loot certain items
--
local function NigglesDelayAutoLoot()
  -- Check the option is enabled
  if ((NigglesDBC.settings.UiAutoLootArch  == true) or
      (NigglesDBC.settings.UiAutoLootFish  == true) or
      (NigglesDBC.settings.UiAutoLootMoney == true)) then
    -- Create a timer to auto-loot certain items
    NigglesTimerCreate("AutoLoot", NigglesAutoLoot, nil, nil, 0.15, false);
  end

  return;
end

--
-- Function to delete an item in a bag or bank slot with a modified click.
-- The deletion is confirmed by calling the appropriate Blizzard function.
--
local function NigglesItemDelete(bagId, slotId)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiItemDelete == true) then
    -- Get info for the item clicked on
    local itemId = GetContainerItemID(bagId, slotId);

    -- Check the click isn't already handled
    if ((itemId) and
        (IsModifiedClick("NIGGLESITEMDELETE"))) then
      -- Pick up the item clicked on
      PickupContainerItem(bagId, slotId);

      -- Get the quality of the item clicked on
      local itemName, _, itemQuality = GetItemInfo(itemId);

      -- Trigger the standard delete item action
      UIParent_OnEvent(self, "DELETE_ITEM_CONFIRM", itemName, itemQuality);
    end
  end

  return;
end

--
-- Function to delete an item in a bag with a modified click.
--
local function NigglesBagItemClicked(self, button)
  -- Delete the item
  NigglesItemDelete(self:GetParent():GetID(), self:GetID());

  return;
end

--
-- Function to delete an item in a bank slot with a modified click.
--
local function NigglesBankItemClicked(self, button)
  -- Delete the item
  NigglesItemDelete(BANK_CONTAINER, self:GetID());

  return;
end

--
-- Function to switch to the first page of a merchant's wares when the 'Prev'
-- button is right-clicked.
--
local function NigglesMerchantFirst(self, button)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiFirstLastPage == true) then
    -- Check the right button was clicked and the 'Prev' button is enabled
    if ((button == "RightButton") and
        (MerchantPrevPageButton:IsEnabled())) then
      MerchantFrame.page = 1;
      MerchantFrame_Update();
      PlaySound("igMainMenuOptionCheckBoxOn");
    end
  end

  return;
end

--
-- Function to switch to the last page of a merchant's wares when the 'Next'
-- button is right-clicked.
--
local function NigglesMerchantLast(self, button)
  -- Local Variables
  local numItems = GetMerchantNumItems();

  -- Check the option is enabled
  if (NigglesDBC.settings.UiFirstLastPage == true) then
    -- Check the right button was clicked and the 'Next' button is enabled
    if ((button == "RightButton") and
        (MerchantNextPageButton:IsEnabled())) then
      -- Change to the last page
      MerchantFrame.page = math.ceil(numItems/MERCHANT_ITEMS_PER_PAGE);
      MerchantFrame_Update();
      PlaySound("igMainMenuOptionCheckBoxOn");
    end
  end

  return;
end

--
-- Function to zoom the minimap all the way in when the button is
-- right-clicked.
--
local function NigglesMinimapZoomIn(self, button)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiMiniMapZoom == true) then
    -- Check the right button was clicked and the 'Zoom In' button is enabled
    if ((button == "RightButton") and
        (MinimapZoomIn:IsEnabled())) then
      MinimapZoomIn:Disable();
      MinimapZoomOut:Enable();
      PlaySound("igMinimapZoomIn");
      Minimap:SetZoom(Minimap:GetZoomLevels()-1);
    end
  end

  return;
end

--
-- Function to zoom the minimap all the way out when the button is
-- right-clicked.
--
local function NigglesMinimapZoomOut(self, button)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiMiniMapZoom == true) then
    -- Check the right button was clicked and the 'Zoom Out' button is enabled
    if ((button == "RightButton") and
        (MinimapZoomOut:IsEnabled())) then
      MinimapZoomIn:Enable();
      MinimapZoomOut:Disable();
      PlaySound("igMinimapZoomOut");
      Minimap:SetZoom(0);
    end
  end

  return;
end

--
-- Function to toggle Equipment Manager Ignore for a item slot.
-- This function implements functionality from the function
-- 'PaperDollFrameItemFlyoutButton_OnClick' in PaperDollFrame.lua.
--
local function NigglesPaperDollItemSlotClicked(self)
  -- Check the option is enabled
  if (NigglesDBC.settings.UiEMIgnoreToggle == true) then
    -- Check the correct modifier key is down and Equipment Manager is shown
    if ((IsAltKeyDown()) and
        ((PaperDollEquipmentManagerPane:IsShown() and
         (PaperDollEquipmentManagerPane.selectedSetName)) or
         (GearManagerDialogPopup:IsShown())) and
         (self.popoutButton:IsShown())) then
      -- Get the ID of the slot
      local slotId = self:GetID();

      -- Toggle the ignoring of the slot
      if (EquipmentManagerIsSlotIgnoredForSave(slotId)) then
        EquipmentManagerUnignoreSlotForSave(slotId);
        self.ignored = nil;
      else
        EquipmentManagerIgnoreSlotForSave(slotId);
        self.ignored = true;
      end

      -- Update the slot and the flyout menu
      PaperDollItemSlotButton_Update(self);
      EquipmentFlyout_Show(self);
      PaperDollEquipmentManagerPaneSaveSet:Enable();
    end
  end

  return;
end

--
-- Function to switch to the first page of the spellbook when the 'Prev'
-- button is right-clicked.
--
local function NigglesSpellBookFirst(self, button)
  -- Local variables
  local currentPage;

  -- Check the option is enabled
  if (NigglesDBC.settings.UiFirstLastPage == true) then
    -- Check the right button was clicked and the 'Prev' button is enabled
    if ((button == "RightButton") and
        (SpellBookPrevPageButton:IsEnabled())) then
      -- Get the current page
      currentPage = SpellBook_GetCurrentPage();

      -- Check the spell book isn't already on the first page
      if (currentPage > 1) then
        -- Change the page number to page 2
        if (SpellBookFrame.bookType == BOOKTYPE_SPELL) then
          SPELLBOOK_PAGENUMBERS[SpellBookFrame.selectedSkillLine] = 2;
        else
          SPELLBOOK_PAGENUMBERS[SpellBookFrame.bookType] = 2;
        end

        -- Call the function for the Next button to change to the last page
        SpellBookPrevPageButton_OnClick();
      end
    end
  end

  return;
end

--
-- Function to switch to the last page of the spellbook when the 'Next'
-- button is right-clicked.
--
local function NigglesSpellBookLast(self, button)
  -- Local variables
  local currentPage;
  local maxPages;

  -- Check the option is enabled
  if (NigglesDBC.settings.UiFirstLastPage == true) then
    -- Check the right button was clicked and the 'Next' button is enabled
    if ((button == "RightButton") and
        (SpellBookNextPageButton:IsEnabled())) then
      -- Get the current and maximum pages
      currentPage, maxPages = SpellBook_GetCurrentPage();

      -- Check the spellbook isn't already on the last page
      if (currentPage < maxPages) then
        -- Change the page number to the penultimate page
        if (SpellBookFrame.bookType == BOOKTYPE_SPELL) then
          SPELLBOOK_PAGENUMBERS[SpellBookFrame.selectedSkillLine] = maxPages-1;
        else
          SPELLBOOK_PAGENUMBERS[SpellBookFrame.bookType] = maxPages-1;
        end

        -- Call the function for the Next button to change to the last page
        SpellBookNextPageButton_OnClick();
      end
    end
  end

  return;
end

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to automatically loot certain items
--
function NigglesAutoLoot()
  -- Local Variables
  local numLootItems = GetNumLootItems();
  local isLocked;
  local itemName;
  local itemLink;
  local itemType;

  -- Check the loot frame is shown
  if (LootFrame:IsShown()) then
    -- Check if the loot is from fishing
    if ((NigglesDBC.settings.UiAutoLootFish == true) and
        (IsFishingLoot())) then
      -- Loot all items
      for slotIdx = 1, numLootItems do
        _, _, _, _, _, _, _, isLocked = GetLootSlotInfo(slotIdx);
        if (not isLocked) then
          LootSlot(slotIdx);
        end
      end
    else
      -- Loot any appropriate items
      for slotIdx = 1, numLootItems do
        _, itemName, _, _, _, _, _, isLocked = GetLootSlotInfo(slotIdx);
        if (not isLocked) then
          itemType = GetLootSlotType(slotIdx);
          itemLink = GetLootSlotLink(slotIdx);
          if (itemType == LOOT_SLOT_MONEY) then
            if (NigglesDBC.settings.UiAutoLootMoney == true) then
              LootSlot(slotIdx);
            end
          elseif ((NigglesDBC.settings.UiAutoLootArch == true) and
                  (((itemType == LOOT_SLOT_CURRENCY) and
                    (NigglesArchaeologyIsFragment(itemName))) or
                   ((itemType == LOOT_SLOT_ITEM) and
                    (NigglesArchaeologyIsKeystone(itemLink))))) then
            LootSlot(slotIdx);
          end
        end
      end
    end
  end

  return;
end

--
-- Function to toggle all the class colour checkboxes in the Chat 'General
-- Config' panel
--
function NigglesChatConfigOnClick(button)
  -- Local variables
  local checkBox;
  local checked = false;
  local checkIdx = 1;
  local frame = button:GetParent();
  local frameName = frame:GetName();

  -- Work out what state to set the checkboxes to
  checkBox = _G[frameName.."CheckBox"..checkIdx.."ColorClasses"];
  while (checkBox ~= nil) do
    -- Check if the checkbox is visible
    if ((checkBox:IsVisible()) and
        (not checkBox:GetChecked())) then
      checked = true;
      break;
    end

    -- Move on to the next checkbox
    checkIdx = checkIdx+1;
    checkBox = _G[frameName.."CheckBox"..checkIdx.."ColorClasses"];
  end

  -- Process each of the class color checkboxes
  checkIdx = 1;
  checkBox = _G[frameName.."CheckBox"..checkIdx.."ColorClasses"];
  while (checkBox ~= nil) do
    -- Check if the checkbox is visible
    if (checkBox:IsVisible()) then
      checkBox:func(checked);
      checkBox:SetChecked(checked);
    end

    -- Move on to the next checkbox
    checkIdx = checkIdx+1;
    checkBox = _G[frameName.."CheckBox"..checkIdx.."ColorClasses"];
  end

  return;
end

--
-- Function to hook into the standard UI
--
-- NOTE: This function should only ever be called once
--
function NigglesHookUi()
  -- Hook functions into the Zoom buttons on the Minimap
  MinimapZoomIn:HookScript("OnMouseUp", NigglesMinimapZoomIn);
  MinimapZoomOut:HookScript("OnMouseUp", NigglesMinimapZoomOut);

  -- Hook a function into 'Raid' frame to set the targets for the buttons
  RaidFrame:HookScript("OnShow", NigglesRaidButtonsSetTargets);

  -- Hook a function into 'Loot' frame to automatically loot certain items
  LootFrame:HookScript("OnShow", NigglesDelayAutoLoot);

  -- Hook the clicking of any container item button
  hooksecurefunc("ContainerFrameItemButton_OnModifiedClick",
    NigglesBagItemClicked);
  hooksecurefunc("BankFrameItemButtonGeneric_OnModifiedClick",
    NigglesBankItemClicked);

  -- Hook functions into the 'Prev' and 'Next' buttons in the 'Spellbook' frame
  SpellBookPrevPageButton:HookScript("OnMouseUp", NigglesSpellBookFirst);
  SpellBookNextPageButton:HookScript("OnMouseUp", NigglesSpellBookLast);

  -- Hook functions into the 'Merchant' frame
  MerchantPrevPageButton:HookScript("OnMouseUp", NigglesMerchantFirst);
  MerchantNextPageButton:HookScript("OnMouseUp", NigglesMerchantLast);
  hooksecurefunc("MerchantFrame_Update", NigglesMerchantSearchUpdate);

  -- Hook functions into the 'Stack Split' frame
  StackSplitFrame:HookScript("OnShow", NigglesStackSplitOnShow);

  -- Hook a function into the equipment buttons
  hooksecurefunc("PaperDollItemSlotButton_OnModifiedClick",
    NigglesPaperDollItemSlotClicked);

  -- Hook into the Game Time frame
  GameTimeFrame:HookScript("OnClick", NigglesCalendarEventsOnClick);

  -- Hook into the creation of the Achievement Alert frames
  hooksecurefunc("AchievementAlertFrame_OnLoad",
    NigglesAchievementsAlertOnLoad);

  return;
end

--
-- Function to set the targets for the buttons in the Raid panel
--
function NigglesRaidButtonsSetTargets()
  -- Local variables
  local button;
  local unitName;

  -- Check the player is in a raid
  if (IsInRaid()) then
    -- Set the target for all of the buttons
    for buttonIdx = 1, MAX_RAID_MEMBERS do
      button = _G["RaidGroupButton"..buttonIdx];
      if (button ~= nil) then
        unitName = _G[button:GetName().."Name"]:GetText();
        if (InCombatLockdown() == nil) then
          if ((unitName ~= nil) and
              (NigglesDBC.settings.RaidsTarget == true)) then
            button:SetAttribute("type", "target");
            button:SetAttribute("unit", unitName);
          else
            button:SetAttribute("type", nil);
            button:SetAttribute("unit", nil);
          end
        else
          Niggles.updateRaid = true;
        end
      end
    end
  end

  return;
end

--
-- Function to process clicking of Niggles Stack Split 'Maximum' button
--
function NigglesStackSplitMaxOnClick()
  -- Hide the 'Split Stack' frame
  StackSplitFrame:Hide();
  
  -- Check the 'Split Stack' frame has an owner
  if (StackSplitFrame.owner) then
    -- Call the owner's 'Split Stack' function, with max stack size
    StackSplitFrame.owner.SplitStack(StackSplitFrame.owner,
      StackSplitFrame.maxStack);
  end

  return;
end

--
-- Function called when the 'Stack Split' frame is shown
--
function NigglesStackSplitOnShow()
  -- Local Variables
  local owner;

  -- Find the ultimate owner of the 'Stack Split' frame
  owner = StackSplitFrame.owner;
  while ((owner ~= nil) and
         (owner ~= MerchantFrame) and
         (owner ~= UIParent)) do
    owner = owner:GetParent();
  end

  -- Show/Hide the Niggles 'Stack Split' frame, depending on the owner
  if ((NigglesDBC.settings.UiMerchantBuyMax == true) and
      (owner == MerchantFrame) and
      (StackSplitFrame.maxStack > 1)) then
    -- Create the Niggles 'Stack Split' frame, if required
    if (NigglesStackSplit == nil) then
      -- Create the Niggles 'Stack Split' frame
      CreateFrame("Frame", "NigglesStackSplit", StackSplitFrame,
        "NigglesStackSplitTemplate");

      -- Adjust the level of the 'Stack Split' buttons
      StackSplitOkayButton:SetFrameLevel(129);
      StackSplitCancelButton:SetFrameLevel(129);
    end

    -- Set up the Niggles 'Stack Split' frame and show it
    NigglesStackSplitMax:SetText(format("%s (%d)", MAXIMUM,
      StackSplitFrame.maxStack));
    NigglesStackSplitMax:Enable();
    NigglesStackSplit:Show();
  else
    -- Hide the Niggles 'Stack Split' frame, if it exists
    if (NigglesStackSplit ~= nil) then
      NigglesStackSplit:Hide();
    end
  end

  return;
end
